import React, { useEffect, useState } from 'react';
import useIsInViewport from 'use-is-in-viewport';
export default function WaypointLazy(props) {
    const { width, height, fallback, threshold } = Object.assign({ width: 400, height: 300, fallback: null, threshold: 0.1 }, props);
    const [state, setState] = useState({ last: null, seen: false, play: false });
    const [isInViewport, childRef] = useIsInViewport({ threshold });
    useEffect(() => {
        if (props.onVisibleChange) { // && isInViewport !== state.last) { // hook only triggers on isInViewport
            // Make sure we don't fire 'no visibility' when the element has not been seen yet
            const isNotVisibleAndNotSeenYet = !isInViewport && !state.seen;
            if (!isNotVisibleAndNotSeenYet) {
                props.onVisibleChange(isInViewport ? true : false);
                // setState((p: any) => ({ ...p, last: isInViewport }));
            }
        }
        if (isInViewport && !state.seen) {
            setState((p) => ({ ...p, play: true, seen: true }));
        }
        else if (!isInViewport && state.seen) {
            setState((p) => ({ ...p, play: false }));
        }
    }, [isInViewport]);
    // const Lazy = React.useMemo( () => React.lazy(f), [f] );
    return (React.createElement(React.Suspense, { fallback: fallback },
        React.createElement("div", { ref: childRef }, state.seen ? React.createElement(React.Fragment, null, props.children) : React.createElement("div", { style: { width, height } }))));
}
/*
export default function WaypointLazySuspend(props: Props) {
  const {width, height, fallback} = Object.assign({width:400, height:300, fallback:null}, props);
  return (<React.Suspense fallback={ null }> <WaypointLazy { ...props } /> </React.Suspense>);
}
*/
//# sourceMappingURL=index.js.map